from __future__ import absolute_import
import falcon
import argparse
import time
import sys
import json

from netq_lib.orm.redisdb import (
    set_connection_settings, reset_connection_settings,
    are_conn_and_listener_alive, BackendExceptions
)

from netq_lib.common import utils
from netq_apps.modules.core.trace.trace import (
    path_trace_mac,
    path_trace_route
)

from netq_backend.lib.parser import (
    get_request_data,
    get_time
)

from netq_lib.common.enums import ConfKw
from netq_lib.common import utils
from netq_apps.lib.netq import NetQ

class NetqTrace(object):

    def __init__(self):
        try:
            conf_dict = utils.get_yaml_config('netq')
            backend = conf_dict[ConfKw.BACKEND]
            self.netq = NetQ(backend, None, None)
        except Exception as e:
            self.netq = None
        return

    def parse_request(self, req_data):
        (self.start_time, _, _) = get_time(req_data)
        if not self.start_time:
            self.start_time = int(time.time())

        self.vrf = str(req_data.get('vrf', 'default'))

        self.source = req_data.get('source', None )
        if self.source:
            self.source = str(self.source)

        self.vlan = str(req_data.get('vlan', '*'))

        self.destination = req_data.get('destination', None)
        if self.destination:
            self.destination = str(self.destination)

        self.hostname = req_data.get('hostname', None)
        if self.hostname:
            self.hostname = str(self.hostname)


    def on_post(self, req, resp):
        '''
        Expect request format:
            {
               'hostname': host
               'start_time': <utc>,
               'end_time': <utc>,
               'time_inverals': in seconds,
               'source': <host, ip, mac>,
               'destination' : <host, ip, mac>.
               'vrf' : <vrf>
            }
        '''
        try:
            req_data = get_request_data(req)
        except Exception as e:
            resp.status = falcon.HTTP_400
            resp.content_type = 'application/json'
            resp.data = json.dumps('Invalid POST data')
            return

        self.parse_request(req_data)

        if not self.source:
            resp.status = falcon.HTTP_400
            resp.content_type = 'application/json'
            resp.data = json.dumps('Missing trace Source')
            return

        if not self.destination:
            resp.status = falcon.HTTP_400
            resp.content_type = 'application/json'
            resp.data = json.dumps('Missing trace destination')
            return

        visited_nodes = set()
        if ':' in self.destination:
            output, error = path_trace_mac(self.netq, self.destination,
                                           self.vlan,
                                           self.hostname,
                                           None,
                                           self.vrf,
                                           0, True, visited_nodes)
        else:
            output, error = path_trace_route(self.netq, self.destination,
                                             None,
                                             self.source,
                                             self.vrf,
                                             0, True, visited_nodes,
                                             None, None, None)
        if not output:
            resp_data = {
                'request': req_data,
                'totalPaths': 0,
                'error': 'No path to %s from %s found' % (self.source, self.destination)
            }
        else:
            flattend_paths = []
            utils.prtree(output[0], 0, True, [], flattend_paths)
            total_paths = len(flattend_paths)
            output_error = []
            for estr in error.split('\n'):
                if estr and estr not in output_error:
                    output_error.append(estr)

            resp_data = {
                'request': req_data,
                'totalPaths': total_paths,
                'error': output_error,
                'path_mtu': str(output[0].get('pmtu', 'N/A')),
                'paths': flattend_paths
            }
        resp.data = json.dumps(resp_data)
        resp.status = falcon.HTTP_200
        resp.content_type = 'application/json'
        return

