# Copyright 2016, 2017 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

"""ROM models
"""
import inspect
from influxdb import InfluxDBClient

class EventDatabaseException(Exception):
    pass

class EventFieldException(Exception):
    pass

class EventObjectException(Exception):
    pass

EventType = [ 'Bgp', 'Ospf', 'Link', 'Vlan', 'Clag', 'Sensors', 'Service' ]
EventSeverity = [ 'CRITICAL', 'WARNING', 'INFO', 'DEBUG',  ]

class EventObject(object):
    timestamp = int
    eventType = str
    severity = str
    message = str

    def __init__(self, **kwargs):
        for key, value in self.__class__.__dict__.items():
            if not isinstance(value, type):
                continue

            if key in kwargs:
                if not isinstance(kwargs[key], value):
                    errorString = "{} should be {}".format(key, value)
                    raise EventFieldException(errorString)
                self.__dict__[key] = value(kwargs[key])
        return

    def json_output(self):
        value = {}
        value['timestamp'] = getattr(self, 'timestamp')
        value['eventType'] = getattr(self, 'eventType')
        value['severity'] = getattr(self, 'severity')
        value['message'] = getattr(self, 'message')
        return "{}".format(value)

    def save(self, database):
        try:
            database.save(self)
        except Exception as e:
            raise EventObjectException("database save operation failed")

    def tags(self):
        value = {}
        value['severity'] = getattr(self, 'severity')
        value['eventType'] = getattr(self, 'eventType')
        return value

    def fields(self):
        value = {}
        value['message'] = getattr(self, 'message')
        return value

    def time(self):
        return getattr(self, 'timestamp')


class EventDatabase(object):
    """Class to interface with Event database"""
    __DB_NAME = 'netqEventDatabase'
    __MEASUREMENT = 'netqEvent'

    def __init__(self, **kwargs):
        try:
            self.client = InfluxDBClient(database=self.__DB_NAME)
            self.client.create_database(self.__DB_NAME)
        except Exception as e:
            raise EventDatabaseException("Failed to connect to influxdb")

        self.dataPoints = []
        return

    def save(self, event):
        dataPoint = {}
        dataPoint["measurement"] = self.__MEASUREMENT
        dataPoint["tags"] = event.tags()
        dataPoint["fields"] = event.fields()
        dataPoint["time"] = event.time()
        self.dataPoints.append(dataPoint)

    def commit(self):
        if not self.dataPoints:
            return

        try:
            self.client.write_points(self.dataPoints)
        except Exception as e:
            raise EventDatabaseException("Failed to commit data points to influxdb")
        self.dataPoints = []
        return

    def query(self, queryString):
        result = self.client.query(queryString)
        for datapoint in result.get_points():
            yield datapoint

    def raw_count(self, queryString):
        value = 0
        result = self.client.query(queryString)
        for datapoint in result.get_points():
            if 'count_message' in datapoint:
                value = datapoint['count_message']
        return value

    def event_count(self, startTime, endTime, interval, filters={}):
        eventTypeFilter = filters.get('eventType', None)
        severityFilter = filters.get('severity', None)

        data = {}
        startTimeList = [ ts for ts in range(startTime, endTime, interval) ]
        data['startTimes'] = startTimeList

        eventCounts = {}
        for eventType in EventType:
            # If filter is defined, include only the event_types specified 
            # in the filter (it's an include filter)
            if eventTypeFilter and eventType not in eventTypeFilter:
                continue

            counterList = []
            for ts in startTimeList:
                counters = {}
                total = 0

                for severity in EventSeverity:
                    if severityFilter and severity not in severityFilter:
                        continue

                    queryString = "select count(*) from {} where eventType = \'{}\' and\
                            severity = \'{}\' and time > {} and time <= {}".format(
                                self.__MEASUREMENT, eventType, severity, ts, ts + interval)

                    counters[severity] = self.raw_count(queryString)
                    total = total + counters[severity]
                counters['total'] = total
                counterList.append(counters)
            eventCounts[eventType] = counterList
        data['eventCounts'] = eventCounts
        return data

    def event_messages(self, startTime, endTime, filters={}):
        eventTypeFilter = filters.get('eventType', None)
        severityFilter = filters.get('severity', None)

        event_messages_list = []
        for eventType in EventType:
            # If filter is defined, include only the event_types specified 
            # in the filter (it's an include filter)
            if eventTypeFilter and eventType not in eventTypeFilter:
                continue
            for severity in EventSeverity:
                if severityFilter and severity not in severityFilter:
                    continue
                queryString = "select * from {} where eventType = \'{}\' and\
                                severity = \'{}\' and time > {} and time <= {}".format(
                                self.__MEASUREMENT,  eventType, severity, startTime, endTime)
                for dataPoint in self.query(queryString):
                    event_messages_list.append(dataPoint)
        return event_messages_list

if __name__ == '__main__':
    db = EventDatabase()
    e = EventObject(timestamp=1001, eventType='Bgp', severity='CRITICAL', message='Session down')
    db.save(e)
    e = EventObject(timestamp=1002, eventType='Bgp', severity='WARNING', message='Session down')
    db.save(e)
    db.commit()
    print db.event_count(1, 2000, 1000)
    print db.event_count(1, 2000, 1000, {'severity': 'WARNING'})
    print db.event_messages(1, 2000, {'eventType': ['Bgp'], 'severity': ['CRITICAL']})
